/*  
	Extension:	Wikipedia formatting context menu for Mozilla based browsers
	Author: 	Bananeweizen, http://www.bananeweizen.de
	Based on:	bbcode extension of Jed Brown, http://jedbrown.net/
				PastQuote extension by Ben Basson (Cusser), http://cusser.net/extensions/
*/

var gPreferences;
var wikipedia_stringBundle;
var wikipedia_Document = null;

var wikipedia = {
	init: function() {
		wikipedia_setupDocument(document);
		try{
			var menu = document.getElementById('contentAreaContextMenu');
			menu.addEventListener('popupshowing', wikipedia.showHide, false);
		}catch (e){
			//Foobar!
		}
	},
	
	showHide: function() {
		document.getElementById('context-wikipedia').hidden = document.getElementById('context-undo').hidden;
//		check    window.content.location.toString()
	},

	wikipedia: function(myCommand, forceSelection) {
		var clip = Components.classes["@mozilla.org/widget/clipboard;1"].createInstance(Components.interfaces.nsIClipboard);
		if (!clip){
			return false;
		}
		
		var trans = Components.classes["@mozilla.org/widget/transferable;1"].createInstance(Components.interfaces.nsITransferable);
		if (!trans){
			return false;
		}
		
		trans.addDataFlavor("text/unicode");
		clip.getData(trans,clip.kGlobalClipboard);
		
		var str = null;
		var str_clipboard=new Object();
		var strLength=new Object();
		try{
			trans.getTransferData("text/unicode",str_clipboard,strLength);
			if (str_clipboard){
				str_clipboard=str_clipboard.value.QueryInterface(Components.interfaces.nsISupportsString);
			}
			if (str_clipboard){
				pastetext=str_clipboard.data.substring(0,strLength.value / 2);
			}
		}catch (e){
			//alert("No text in the clipboard, please copy something first.");
		}
    
		var theBox = document.commandDispatcher.focusedElement;
		var oPosition = theBox.scrollTop;
		var oHeight = theBox.scrollHeight;

		//Get Selected text (if selected)
		var startPos = theBox.selectionStart;
		var endPos = theBox.selectionEnd;
		str = theBox.value.substring(startPos, endPos);
		
		//check for mandatory input
		if (forceSelection) {
			if (str==null || str=="") {
				str=getInput("wikipedia.input."+myCommand)
				if (str==null || str=="") {
					return;
				}
			}
		}

        wikipedia.insertAtCursorSetup(myCommand, str_clipboard, str, theBox);
		var nHeight = theBox.scrollHeight - oHeight;
		theBox.scrollTop = oPosition + nHeight;
	},
  
  	insertAtCursorSetup: function(myCommand, str_clipboard, str, theBox) {
	// Function taken from http://www.alexking.org/blog/2003/06/02/inserting-at-the-cursor-using-javascript/
	// Modified to return cursor to correct place
    switch (myCommand){
		case "link":
			wikipedia.insertLinkAtCursor(theBox, str);
			break;
		
		case "bold":
			wikipedia.insertAtCursor(theBox,"'''" + str + "'''");
			break;

		case "italic":
			wikipedia.insertAtCursor(theBox,"''" + str + "''");
			break;
		    
		case "bolditalic":
			wikipedia.insertAtCursor(theBox,"'''''" + str + "'''''");
			break;
		    
		case "underline":
			wikipedia.insertAtCursor(theBox,"<u>" + str + "</u>");
			break;
		    
		case "strike":
			wikipedia.insertAtCursor(theBox,"<strike>" + str + "<strike>");
			break;
		    
		case "small":
			wikipedia.insertAtCursor(theBox,"<small>" + str + "</small>");
			break;
		
		case "paragraph":
			wikipedia.insertAtCursor(theBox,"\n==" + str + "==\n");
			break;
		    
		case "subparagraph":
			wikipedia.insertAtCursor(theBox,"\n===" + str + "===\n");
			break;
		    
		case "subsubparagraph":
			wikipedia.insertAtCursor(theBox,"\n====" + str + "====\n");
			break;
		    
		case "weblink":
			wikipedia.insertAtCursor(theBox,"[" + str + "]");
			break;
		
		case "list.simple":
			wikipedia.insertList(theBox,"*", str);
			break;
		
		case "list.numbered":
			wikipedia.insertList(theBox,"#",str);
			break;
		
		case "list.definition":
			var topic=wikipedia_stringBundle.getString("wikipedia.definitionlist.topic");
			var definition=wikipedia_stringBundle.getString("wikipedia.definitionlist.definition");
			var def=";"+topic+":"+definition+"\n";
			wikipedia.insertAtCursor(theBox,def+def);
			break;

		case "list.remove":
			wikipedia.removeList(theBox,str);
			break;
		
		
		case "table":
			wikipedia.insertTableAtCursor(theBox, str);
			break;

		case "image":
			wikipedia.insertImageAtCursor(theBox, str);
		break;

		//all other text pieces are just inserted
		default : 
			wikipedia.insertAtCursor(theBox, myCommand);
	}
	},

	insertAtCursor: function(theBox, aText) {
		try {
			var command = "cmd_insertText";
			var controller = document.commandDispatcher.getControllerForCommand(command);
			if (controller && controller.isCommandEnabled(command)) {
				controller = controller.QueryInterface(Components.interfaces.nsICommandController);
				var params = Components.classes["@mozilla.org/embedcomp/command-params;1"];
				params = params.createInstance(Components.interfaces.nsICommandParams);
				params.setStringValue("state_data", aText);
				controller.doCommandWithParams(command, params);
			}
		}catch (e) {
			dump("Can't do cmd_insertText! ");
			dump(e+"\n")
		}
	},
  
	insertList: function(theBox, listSign, str) {
		var lines=str.split("\n");
		var insert=lines[0];
		for (i=1;i<lines.length;i=i+1) {
			insert=insert+"\n"+listSign;
			if (trim(lines[i])!="") {
				insert=insert+lines[i];
			}
		}
		for (i=lines.length;i<3;i=i+1) {
			insert=insert+"\n"+listSign;
		}
	  	wikipedia.insertAtCursor(theBox,insert);
	},
	
	removeList: function(theBox, str) {
		var lines=str.split("\n");
		for (i=0;i<lines.length;i=i+1) {
			lines[i]=lines[i].substr(1,lines[i].length-1);
		}
		var insert=lines[0];
		for (i=1;i<lines.length;i=i+1) {
			insert=insert+"\n"+lines[i];
		}
	  	wikipedia.insertAtCursor(theBox,insert);
	},
	
	insertTableAtCursor: function(theBox, str) {
        var ok = [false];
        var params = [0,0,trim(str),false,false,0,0,0,0,0,0];
        window.openDialog("chrome://wikipedia/content/table.xul", "WikiTable","chrome,modal,centerscreen",ok,params);
        if (ok[0]){
        	var rows=params[0];
        	var columns=params[1];
        	var content=params[2];
        	var border=params[3];
        	var headline=params[4];
        	var table="{|";
        	if (border) {
        		table=table+" border=\"1\"";
        	}
        	table=table+"\n";
        	for (r=0;r<rows;r=r+1) {
        		var cellStart="|";
        		if (headline && (r==0)) {
        			cellStart="!";
        		}
        		for (c=0;c<columns;c=c+1) {
        			table=table+cellStart+content+"\n";
        		}
        		if (r<rows-1) {
	        		table=table+"|-\n";
	        	}
        	}
        	table=table+"|}\n";
			wikipedia.insertAtCursor(theBox,table);
        }
	},
	
	insertImageAtCursor: function(theBox, str) {
        var ok = [false];
        // use selected string as either image name or description
        var imageName="";
        var imageDesc="";
        if (str.indexOf(".")>=0) {
        	imageName=trim(str);
        }
        else {
        	imageDesc=trim(str);
        }
        // call dialog
        var params = [imageName,imageDesc,true,false,false,false,0,false,false,false,0,0,0,0,0,0,0,0,0];
        window.openDialog("chrome://wikipedia/content/image.xul", "WikiImage","chrome,modal,centerscreen",ok,params);
        if (ok[0]){
        	var file=params[0];
        	var description=params[1];
        	var thumb=params[2];
        	var thumbsized=params[3];
        	var framed=params[4];
        	var full=params[5];
        	var pixels=params[6];
        	var left=params[7];
        	var center=params[8];
        	var right=params[9];
        	var image="[["+wikipedia_stringBundle.getString("wikipedia.image.namespace")+":"+file;
        	if (framed) {
        		image=image+"|framed";
        	}
        	else if (thumb) {
        		image=image+"|thumb";
        	}
        	else if (thumbsized) {
        		image=image+"|thumb|"+pixels+"px";
        	}
        	if (left) {
        		image=image+"|left";
        	}
        	else if (right) {
        		image=image+"|right";
        	}
        	else if (center) {
        		image=image+"|none";
        	}
        	image=image+"|"+description;
        	image=image+"]]";
        	if (center) {
        		image="<center>"+image+"</center>";
        	}
			wikipedia.insertAtCursor(theBox,image);
        }
	},
	
	insertLinkAtCursor: function(theBox, str) {
        var ok = [false];
        var params = [trim(str),"",0,0,0,0,0,0];
        window.openDialog("chrome://wikipedia/content/link.xul", "WikiLink","chrome,modal,centerscreen",ok,params);
        if (ok[0]){
        	var target=params[0];
        	var label=params[1];
        	var link="[["+target;
        	if (label!="") {
        		link=link+"|"+label;
        	}
        	link=link+"]]";
			wikipedia.insertAtCursor(theBox,link);
        }
	}
}

function trim(s) {
	if (s==null || s=="") {
		return "";
	}
	while (s.substring(0,1) == ' ') {
		s = s.substring(1,s.length);
	}
	while (s.substring(s.length-1,s.length) == ' ') {
		s = s.substring(0,s.length-1);
	}
	return s;
}

function getInput(textId) {
	var dialogText=wikipedia_stringBundle.getString(textId);
	var newtext = prompt(dialogText);
	return newtext;
}

//http://www.xulplanet.com/references/xpcomref/ifaces/nsIPref.html
function getPrefwikipediaCompose() {
	if (!gPreferences) {
		try {
			gPreferences = Components.classes["@mozilla.org/preferences;1"].getService();
			gPreferences = gPreferences.QueryInterface(Components.interfaces.nsIPref);
		} catch (e) {}
	}
	return gPreferences;
}
//May not be needed, but just incase.
function wikipedia_setupDocument(aDoc)
{
	wikipedia_Document = aDoc;
	wikipedia_stringBundle = aDoc.getElementById("wikipediastrings");
}

//Make sure wikipedia loads on start-up of browser.
window.addEventListener('load', wikipedia.init, false);